@@ -1,4 +1,3 @@
-#!/usr/bin/env python2
 # encoding=utf-8
 # Copyright © 2018 Intel Corporation
 
@@ -46,7 +45,10 @@ def arg_parser():
 
 def parse_test_file(filename, nl_format):
     """Check for any special arguments and return them as a list."""
-    with open(filename) as f:
+    # Disable "universal newlines" mode; we can't directly use `nl_format` as
+    # the `newline` argument, because the "bizarro" test uses something Python
+    # considers invalid.
+    with io.open(filename, newline='') as f:
         for l in f.read().split(nl_format):
             if 'glcpp-args:' in l:
                 return l.split('glcpp-args:')[1].strip().split()
@@ -64,8 +66,9 @@ def test_output(glcpp, filename, expfile, nl_format='\n'):
             stderr=subprocess.STDOUT,
             stdin=subprocess.PIPE)
         actual, _ = proc.communicate(f.read())
+        actual = actual.decode('utf-8')
 
-    with open(expfile, 'rb') as f:
+    with open(expfile, 'r') as f:
         expected = f.read()
 
     if actual == expected:
@@ -119,6 +122,9 @@ def test_unix(args):
             for l in diff:
                 print(l, file=sys.stderr)
 
+    if not total:
+        raise Exception('Could not find any tests.')
+
     print('{}/{}'.format(passed, total), 'tests returned correct results')
     return total == passed
 
@@ -155,6 +161,9 @@ def _replace_test(args, replace):
             for l in diff:
                 print(l, file=sys.stderr)
 
+    if not total:
+        raise Exception('Could not find any tests.')
+
     print('{}/{}'.format(passed, total), 'tests returned correct results')
     return total == passed
 
@@ -197,6 +206,9 @@ def test_valgrind(args):
             print('FAIL')
             print(log, file=sys.stderr)
 
+    if not total:
+        raise Exception('Could not find any tests.')
+
     print('{}/{}'.format(passed, total), 'tests returned correct results')
     return total == passed
 
