@@ -79,6 +79,7 @@ st_get_external_sampler_key(struct st_context *st, struct gl_program *prog)
 
       switch (format) {
       case PIPE_FORMAT_NV12:
+      case PIPE_FORMAT_P010:
       case PIPE_FORMAT_P016:
          key.lower_nv12 |= (1 << unit);
          break;
@@ -98,7 +99,8 @@ st_get_external_sampler_key(struct st_context *st, struct gl_program *prog)
          key.lower_xyuv |= (1 << unit);
          break;
       default:
-         printf("unhandled %u\n", format);
+         printf("mesa: st_get_external_sampler_key: unhandled pipe format %u\n",
+                format);
          break;
       }
    }
@@ -134,36 +136,45 @@ struct st_fp_variant_key
    /** for OpenGL 1.0 on modern hardware */
    GLuint lower_two_sided_color:1;
 
+   GLuint lower_flatshade:1;
+   enum compare_func lower_alpha_func:3;
+
    /** needed for ATI_fragment_shader */
    char texture_targets[MAX_NUM_FRAGMENT_REGISTERS_ATI];
 
    struct st_external_sampler_key external;
-
-   GLuint lower_flatshade:1;
-   enum compare_func lower_alpha_func:3;
 };
 
+/**
+ * Base class for shader variants.
+ */
+struct st_variant
+{
+   /** next in linked list */
+   struct st_variant *next;
+
+   /** st_context from the shader key */
+   struct st_context *st;
+
+   void *driver_shader;
+};
 
 /**
  * Variant of a fragment program.
  */
 struct st_fp_variant
 {
+   struct st_variant base;
+
    /** Parameters which generated this version of fragment program */
    struct st_fp_variant_key key;
 
-   /** Driver's compiled shader */
-   void *driver_shader;
-
    /** For glBitmap variants */
    uint bitmap_sampler;
 
    /** For glDrawPixels variants */
    unsigned drawpix_sampler;
    unsigned pixelmap_sampler;
-
-   /** next in linked list */
-   struct st_fp_variant *next;
 };
 
 
@@ -185,39 +196,27 @@ struct st_common_variant_key
 
    /* for user-defined clip-planes */
    uint8_t lower_ucp;
+
+   /* Whether st_variant::driver_shader is for the draw module,
+    * not for the driver.
+    */
+   bool is_draw_shader;
 };
 
 
 /**
- * This represents a vertex program, especially translated to match
- * the inputs of a particular fragment shader.
+ * Common shader variant.
  */
-struct st_vp_variant
+struct st_common_variant
 {
-   /* Parameters which generated this translated version of a vertex
-    * shader:
-    */
+   struct st_variant base;
+
+   /* Parameters which generated this variant. */
    struct st_common_variant_key key;
 
-   /**
-    * The shader variant saved for the draw module to later emulate
-    * selection/feedback/rasterpos.
+   /* Bitfield of VERT_BIT_* bits matching vertex shader inputs,
+    * but not include the high part of doubles.
     */
-   const struct tgsi_token *tokens;
-
-   /** Driver's compiled shader */
-   void *driver_shader;
-
-   /** For using our private draw module (glRasterPos) */
-   struct draw_vertex_shader *draw_shader;
-
-   /** Next in linked list */
-   struct st_vp_variant *next;  
-
-   /** similar to that in st_vertex_program, but with edgeflags info too */
-   GLuint num_inputs;
-
-   /** Bitfield of VERT_BIT_* bits of mesa vertex processing inputs */
    GLbitfield vert_attrib_mask;
 };
 
@@ -225,16 +224,25 @@ struct st_vp_variant
 /**
  * Derived from Mesa gl_program:
  */
-struct st_vertex_program
+struct st_program
 {
-   struct gl_program Base;  /**< The Mesa vertex program */
+   struct gl_program Base;
    struct pipe_shader_state state;
    struct glsl_to_tgsi_visitor* glsl_to_tgsi;
+   struct ati_fragment_shader *ati_fs;
    uint64_t affected_states; /**< ST_NEW_* flags to mark dirty when binding */
 
    /* used when bypassing glsl_to_tgsi: */
    struct gl_shader_program *shader_program;
 
+   struct st_variant *variants;
+};
+
+
+struct st_vertex_program
+{
+   struct st_program Base;
+
    /** maps a TGSI input index back to a Mesa VERT_ATTRIB_x */
    ubyte index_to_input[PIPE_MAX_ATTRIBS];
    ubyte num_inputs;
@@ -243,80 +251,37 @@ struct st_vertex_program
 
    /** Maps VARYING_SLOT_x to slot */
    ubyte result_to_output[VARYING_SLOT_MAX];
-
-   /** List of translated variants of this vertex program.
-    */
-   struct st_vp_variant *variants;
 };
 
 
-/**
- * Geometry program variant.
- */
-struct st_common_variant
+static inline struct st_program *
+st_program( struct gl_program *cp )
 {
-   /* Parameters which generated this variant. */
-   struct st_common_variant_key key;
-
-   void *driver_shader;
-
-   struct st_common_variant *next;
-};
-
-
-/**
- * Derived from Mesa gl_program:
- */
-struct st_common_program
-{
-   struct gl_program Base;
-   struct pipe_shader_state state;
-   struct glsl_to_tgsi_visitor* glsl_to_tgsi;
-   struct ati_fragment_shader *ati_fs;
-   uint64_t affected_states; /**< ST_NEW_* flags to mark dirty when binding */
-
-  /* used when bypassing glsl_to_tgsi: */
-   struct gl_shader_program *shader_program;
-
-   union {
-      struct st_common_variant *variants;
-      struct st_fp_variant *fp_variants;
-   };
-};
-
-
-static inline struct st_vertex_program *
-st_vertex_program( struct gl_program *vp )
-{
-   return (struct st_vertex_program *)vp;
-}
-
-static inline struct st_common_program *
-st_common_program( struct gl_program *cp )
-{
-   return (struct st_common_program *)cp;
-}
-
-static inline void
-st_reference_vertprog(struct st_context *st,
-                      struct st_vertex_program **ptr,
-                      struct st_vertex_program *prog)
-{
-   _mesa_reference_program(st->ctx,
-                           (struct gl_program **) ptr,
-                           (struct gl_program *) prog);
+   return (struct st_program *)cp;
 }
 
 static inline void
 st_reference_prog(struct st_context *st,
-                  struct st_common_program **ptr,
-                  struct st_common_program *prog)
+                  struct st_program **ptr,
+                  struct st_program *prog)
 {
    _mesa_reference_program(st->ctx,
                            (struct gl_program **) ptr,
                            (struct gl_program *) prog);
 }
 
+static inline struct st_common_variant *
+st_common_variant(struct st_variant *v)
+{
+   return (struct st_common_variant*)v;
+}
+
+static inline struct st_fp_variant *
+st_fp_variant(struct st_variant *v)
+{
+   return (struct st_fp_variant*)v;
+}
+
 /**
  * This defines mapping from Mesa VARYING_SLOTs to TGSI GENERIC slots.
  */
@@ -330,32 +295,27 @@ st_get_generic_varying_index(struct st_context *st, GLuint attr)
 extern void
 st_set_prog_affected_state_flags(struct gl_program *prog);
 
-extern struct st_vp_variant *
+extern struct st_common_variant *
 st_get_vp_variant(struct st_context *st,
-                  struct st_vertex_program *stvp,
+                  struct st_program *stvp,
                   const struct st_common_variant_key *key);
 
 
 extern struct st_fp_variant *
 st_get_fp_variant(struct st_context *st,
-                  struct st_common_program *stfp,
+                  struct st_program *stfp,
                   const struct st_fp_variant_key *key);
 
-extern struct st_common_variant *
+extern struct st_variant *
 st_get_common_variant(struct st_context *st,
-                      struct st_common_program *p,
+                      struct st_program *p,
                       const struct st_common_variant_key *key);
 
 extern void
-st_release_vp_variants( struct st_context *st,
-                        struct st_vertex_program *stvp );
+st_release_variants(struct st_context *st, struct st_program *p);
 
 extern void
-st_release_fp_variants( struct st_context *st,
-                        struct st_common_program *stfp );
-
-extern void
-st_release_common_variants(struct st_context *st, struct st_common_program *p);
+st_release_program(struct st_context *st, struct st_program **p);
 
 extern void
 st_destroy_program_variants(struct st_context *st);
@@ -364,29 +324,25 @@ extern void
 st_finalize_nir_before_variants(struct nir_shader *nir);
 
 extern void
-st_prepare_vertex_program(struct st_vertex_program *stvp);
+st_prepare_vertex_program(struct st_program *stvp);
 
 extern void
 st_translate_stream_output_info(struct gl_program *prog);
 
 extern bool
 st_translate_vertex_program(struct st_context *st,
-                            struct st_vertex_program *stvp);
+                            struct st_program *stvp);
 
 extern bool
 st_translate_fragment_program(struct st_context *st,
-                              struct st_common_program *stfp);
+                              struct st_program *stfp);
 
 extern bool
 st_translate_common_program(struct st_context *st,
-                            struct st_common_program *stcp);
+                            struct st_program *stp);
 
 extern void
-st_print_current_vertex_program(void);
-
-extern void
-st_precompile_shader_variant(struct st_context *st,
-                             struct gl_program *prog);
+st_finalize_program(struct st_context *st, struct gl_program *prog);
 
 #ifdef __cplusplus
 }
